@@ -10,7 +10,7 @@ import re
 import ssl
 from pathlib import Path
 from basic_logtools.filelog import LogFile
-from asyncio import shield
+from asyncio import shield, wait_for
 # These values are constant
 try:
     from conf.socket_conf import AF_TYPE, SOCK_TYPE
@@ -396,7 +396,7 @@ class GNCSocket:
         reader.feed_eof()
 
 
-    def heart_beat(self, idc):
+    async def heart_beat(self, idc):
         tnow = now()
         if idc in self.clients.keys():
             reader = self.clients[idc]['reader']
@@ -412,12 +412,35 @@ class GNCSocket:
                 print(msg_error)
                 self.logger.error(msg_error)
                 self.logger.error("no heart_beat, at %s" %tnow)
-                self.close(idc)
+                try:
+                    await wait_for(self.close(idc), timeout=10)
+                except asyncio.TimeoutError as te:
+                    self.logger.exception("Tiempo fuera en intento de cerrar conexión %s, mode %s" %(te, self.mode))
+                    await asyncio.sleep(10)
+                except (ConnectionResetError, ConnectionAbortedError) as conn_error:
+                    self.logger.exception("Excepción por desconexión %s, mode %s"%(conn_error, self.mode))
+                    await asyncio.sleep(10)                    
+                except Exception as e:
+                    print("Excepción no considerada, %s"%e)
+                    self.logger.exception("Excepción no considerada %s, mode %s"%(e, self.mode))
+                    await asyncio.sleep(10)
+                self.logger.exception("Cerrando correctamente la conexión, por no heartbeat")
                 self.status = False
                 return False
         else:
             self.logger.error("no heart_beat, idc %s not client,  mode %s" % (idc, self.mode))
-            self.close(idc)
+            try:
+                await wait_for(self.close(idc), timeout=10)
+            except asyncio.TimeoutError as te:
+                self.logger.exception("Tiempo fuera en intento de cerrar conexión %s, mode %s" %(te, self.mode))
+                await asyncio.sleep(10)
+            except (ConnectionResetError, ConnectionAbortedError) as conn_error:
+                self.logger.exception("Excepción por desconexión %s, mode %s"%(conn_error, self.mode))
+                await asyncio.sleep(10)                    
+            except Exception as e:
+                print("Excepción no considerada, %s"%e)
+                self.logger.exception("Excepción no considerada %s, mode %s"%(e, self.mode))
+                await asyncio.sleep(10)
             self.status = False
             return False
         
@@ -427,172 +450,202 @@ class GNCSocket:
             result = await asyncio.wait_for(future, timeout=self.timeout)
             return result
         except asyncio.TimeoutError as te:
-            self.logger.exception("mode %s, Tiempo fuera al leer en readbytes, %s"%(self.mode,te))
+            print("Error Timeout al leer %d bytes en %s" %(n, self.mode))
+            self.logger.exception("Mode %s, Tiempo fuera al leer en readbytes, %s"%(self.mode,te))
             if self.raise_timeout:
                 raise te
+            else:
+                return None
+        except asyncio.IncompleteReadError as ir:
+            print("Incomplete read, mode %s" %self.mode, reader, n)
+            self.logger.exception("Mode  %s, Tiempo fuera al no poder leer en readbytes %s bytes, %s"%(n, self.mode,ir))
+            return None
+        except (ConnectionResetError, ConnectionAbortedError) as conn_error:
+            self.logger.exception("Excepción por desconexión al intentar leer %s"%conn_error)
+            return None
+        except Exception as e:
+            print("Excepción no considerada")
+            self.logger.exception("Excepción no considerada al intentar leer%s"%e)
+            return None
 
     async def recv_msg(self, id_client):
-        reader = self.clients[id_client]['reader']
-        writer = self.clients[id_client]['writer']
-        heartbeat = lambda:self.heart_beat(id_client)        
-        # addr = self.addr
-        bs_0 = 1
-        count = 0
-        b_header = b''
-        idx_recv = ''
-        t = True
-        msg_tot = b''
-        mlist = []
-        n_msgs_idx = 0
-        try:
-            # moment 1 -> get hueader
-            if writer.transport.is_closing():
-                # self.logger.error("La conexión se cerró %s" % writer)
-                raise Exception("Conexión perdida")
+        if id_client in self.clients:
+            reader = self.clients[id_client]['reader']
+            writer = self.clients[id_client]['writer']
+            #heartbeat = await self.heart_beat(id_client)        
+            # addr = self.addr
+            bs_0 = 1
+            count = 0
+            b_header = b''
             idx_recv = ''
-            page = [0, 0]
-            while t:
-                while count < 4 and heartbeat():
-                    if not reader.at_eof():
-                        # moment 2 -> get content message, check if there are more
-                        try:
-                            char_recv = await shield(self.readbytes(reader, bs_0))
-                        except asyncio.TimeoutError as et:
-                            self.logger.exception("Error de conexió timeout %s" %et)
-                            if self.raise_timeout:
-                                raise te
-                            else:
-                                continue                            
-                        except Exception as ex:
-                            raise Exception("Conexión perdida -> %s" % ex)
-                    else:
-                        ex = reader.exception()
-                        raise Exception("Conexión perdida -> EOF")
-                    b_header += char_recv
-                    if char_recv == b" ":
-                        count += 1
-                    if not heartbeat():
-                        ex = reader.exception()
-                        raise Exception(
-                            "Conexión perdida -> Not Peername" % ex)
-                header = b_header.decode(char_code)
-                b_header = b''
-                spaces = [x.start() for x in re.finditer(' ', header)]
-                # bprint("=======Spaces=======")
-                # rprint(spaces)
-                sp_1 = spaces[0]
-                sp_2 = spaces[1]
-                sp_3 = spaces[2]
-                s_page = header[sp_2 + 1:sp_3].split("/")
-                # gprint("HEADER")
-                # rprint(header)
-                # bprint("S PAGE ON SOCKET")
-                # rprint(s_page)
-                a = page[0]
-                b = page[1]
-                page = list(map(hextring2int, s_page))
-                sp_4 = spaces[3]
-                check_header = header[sp_1 + 1:sp_2]
-                # bprint("Check header spaces_0 -> spaces_1")
-                # rprint(check_header)
-                assert self.header == header[sp_1 + 1:
-                                             sp_2], "No es un encabezado correcto"
-                this_idx_recv = header[:sp_1]
-                # bprint("IDX received")
-                # rprint(this_idx_recv)
-                # rprint("Counter......")
-                # bprint(n_msgs_idx)
-                if idx_recv == '':
-                    idx_recv = this_idx_recv
-                    n_msgs_idx = 1
-                else:
-                    n_msgs_idx += 1
-                # bprint("IDX>")
-                # rprint(n_msgs_idx)
-                # bprint(page)
-                assert n_msgs_idx == page[
-                    0], "Error en mensaje, no coincide #idx con pagina " + str(
-                        page[0])
-                lmsg = header[sp_3 + 1:sp_4]
-                #bprint("HEXLength mgs")
-                # rprint(lmsg)
-                #bprint("Length mgs")
-                # rprint(hextring2int(lmsg))
-                bs_1 = hextring2int(lmsg)+1
-                # gprint("BS1")
-                # bprint(bs_1)
-                # moment 3 -> get next slice of msg
-                b_MSG = b''
-                n = 0
-                while n < bs_1:
-                    if heartbeat():
-                        # moment 2 -> get content message, check if there are more
-                        try:
-                            bm = await shield(self.readbytes(reader, 1))
-                        except asyncio.TimeoutError as et:
-                            self.logger.exception("Error de conexió timeout %s" %et)
-                            if self.raise_timeout:
-                                raise te
-                            else:
-                                continue                                                        
-                        except Exception as ex:
-                            self.logger.exception("Error al leer datos %s, con %s" %(ex, b_MSG))
+            t = True
+            msg_tot = b''
+            mlist = []
+            n_msgs_idx = 0
+            try:
+                # moment 1 -> get hueader
+                if writer.transport.is_closing():
+                    # self.logger.error("La conexión se cerró %s" % writer)
+                    raise Exception("Conexión perdida")
+                idx_recv = ''
+                page = [0, 0]
+                while t:
+                    while count < 4 and await self.heart_beat(id_client):
+                        if not reader.at_eof():
+                            char_recv = None
+                            # moment 2 -> get content message, check if there are more
+                            try:
+                                char_recv = await shield(self.readbytes(reader, bs_0))
+                            except asyncio.TimeoutError as te:
+                                self.logger.exception("Tiempo fuera en intento de cerrar conexión %s, mode %s" %(
+                                    te, self.mode))
+                                await asyncio.sleep(10)
+                                continue
+                            except (ConnectionResetError, ConnectionAbortedError) as conn_error:
+                                self.logger.exception("Excepción por desconexión %s, mode %s"%(
+                                    conn_error, self.mode))
+                                await asyncio.sleep(10)
+                                continue
+                            except Exception as ex:
+                                print("Error al leer data -> %s" % ex, char_recv)
+                                continue
+                        else:
+                            ex = reader.exception()
+                            print("Excepcion de lectura -> EOF")
+                        b_header += char_recv
+                        if char_recv == b" ":
+                            count += 1
+                        if not await self.heart_beat(id_client):
+                            ex = reader.exception()
                             raise Exception(
-                                "Error en conexión %s" % ex)
-                    b_MSG += bm
-                    # rprint("MSG--->")
-                    # bprint(b_MSG)
-                    # rprint(bs_1)
-                    # gprint(n)
-                    n += 1
-                l_ender = len(self.ender.encode(char_code))
-                # PROBLEM WITH DECODE BEFORE JOIN
-                # print("===========")
-                # print(b_MSG)
-                # bprint(len(b_MSG))
-                nlast = b_MSG.rfind(b" ")
-                pre_msg = b_MSG[:nlast]
-                #print("Pre msg", pre_msg)
-                #print(pre_msg[0:1], pre_msg[-1::])
-                # if pre_msg[0:1] == h:
-                #    qmsg = pre_msg[1::]
-                # if pre_msg[-1::]==h:
-                #    wmsg = qmsg[:-1]
-                #print("Pre context split")
-                count = 0
-                #print("Msg recibido en socket read_socket %s" % pre_msg)
-                #bprint("page_0 == page_1")
-                #rprint(page[0] == page[1])
-                if pre_msg == b"<END>" or page[0] == page[1]:
-                    # rprint("==========================")
-                    #bprint("Fin de mensaje")
-                    # rprint("==========================")
-                    t = False
-                    mlist.append(pre_msg)
-                    n_msgs_idx = 0
-                    #print("Cerrando", mlist)
-                    break
-                else:
-                    mlist.append(pre_msg)
-                    #print("Added msg ", mlist)
-        except asyncio.CancelledError as ex:
-            self.logger.exception("Error de conexión cancelada  %s" %ex)                        
-            raise ex
-        except socket.error as ex:
-            self.logger.exception("Error de conexión de socket  %s" %ex)                        
-            raise ex
-        except Exception as ex:
-            self.logger.exception("Error de conexión  %s" %ex)            
-            raise ex        
-        #print("Procesando final")
-        msg_tot = b"".join(mlist)
-        #print("MSG JOINED", msg_tot)
-        MSG_TOT = msg_tot.decode(char_code)
-        MT = MSG_TOT.strip()
-        MT = MSG_TOT.rstrip()
-        #print("Fin del mensaje", MSG_TOT)
-        self.msg_r = MSG_TOT
-        return MSG_TOT
+                                "Conexión perdida -> Not Peername" % ex)
+                    header = b_header.decode(char_code)
+                    b_header = b''
+                    spaces = [x.start() for x in re.finditer(' ', header)]
+                    # bprint("=======Spaces=======")
+                    # rprint(spaces)
+                    sp_1 = spaces[0]
+                    sp_2 = spaces[1]
+                    sp_3 = spaces[2]
+                    s_page = header[sp_2 + 1:sp_3].split("/")
+                    # gprint("HEADER")
+                    # rprint(header)
+                    # bprint("S PAGE ON SOCKET")
+                    # rprint(s_page)
+                    a = page[0]
+                    b = page[1]
+                    page = list(map(hextring2int, s_page))
+                    sp_4 = spaces[3]
+                    check_header = header[sp_1 + 1:sp_2]
+                    # bprint("Check header spaces_0 -> spaces_1")
+                    # rprint(check_header)
+                    assert self.header == header[sp_1 + 1:
+                                                 sp_2], "No es un encabezado correcto"
+                    this_idx_recv = header[:sp_1]
+                    # bprint("IDX received")
+                    # rprint(this_idx_recv)
+                    # rprint("Counter......")
+                    # bprint(n_msgs_idx)
+                    if idx_recv == '':
+                        idx_recv = this_idx_recv
+                        n_msgs_idx = 1
+                    else:
+                        n_msgs_idx += 1
+                    # bprint("IDX>")
+                    # rprint(n_msgs_idx)
+                    # bprint(page)
+                    assert n_msgs_idx == page[
+                        0], "Error en mensaje, no coincide #idx con pagina " + str(
+                            page[0])
+                    lmsg = header[sp_3 + 1:sp_4]
+                    #bprint("HEXLength mgs")
+                    # rprint(lmsg)
+                    #bprint("Length mgs")
+                    # rprint(hextring2int(lmsg))
+                    bs_1 = hextring2int(lmsg)+1
+                    # gprint("BS1")
+                    # bprint(bs_1)
+                    # moment 3 -> get next slice of msg
+                    b_MSG = b''
+                    n = 0
+                    while n < bs_1:
+                        heartbeat = await self.heart_beat(id_client)
+                        bm = None
+                        if heartbeat:
+                            # moment 2 -> get content message, check if there are more
+                            try:
+                                bm = await shield(self.readbytes(reader, 1))
+                            except asyncio.TimeoutError as te:
+                                self.logger.exception("Tiempo fuera en intento de cerrar conexión %s, mode %s" %(
+                                    te, self.mode))
+                                await asyncio.sleep(10)
+                                continue
+                            except (ConnectionResetError, ConnectionAbortedError) as conn_error:
+                                self.logger.exception("Excepción por desconexión %s, mode %s"%(
+                                    conn_error, self.mode))
+                                await asyncio.sleep(10)
+                                continue                            
+                            except Exception as ex:
+                                self.logger.exception("Error al leer datos %s, con %s" %(ex, b_MSG))
+                                print("Error en conexión excepction:<%s>" % ex)
+                                print("Mensaje....", bm)
+                                continue
+                        b_MSG += bm
+                        # rprint("MSG--->")
+                        # bprint(b_MSG)
+                        # rprint(bs_1)
+                        # gprint(n)
+                        n += 1
+                    l_ender = len(self.ender.encode(char_code))
+                    # PROBLEM WITH DECODE BEFORE JOIN
+                    # print("===========")
+                    # print(b_MSG)
+                    # bprint(len(b_MSG))
+                    nlast = b_MSG.rfind(b" ")
+                    pre_msg = b_MSG[:nlast]
+                    #print("Pre msg", pre_msg)
+                    #print(pre_msg[0:1], pre_msg[-1::])
+                    # if pre_msg[0:1] == h:
+                    #    qmsg = pre_msg[1::]
+                    # if pre_msg[-1::]==h:
+                    #    wmsg = qmsg[:-1]
+                    #print("Pre context split")
+                    count = 0
+                    #print("Msg recibido en socket read_socket %s" % pre_msg)
+                    #bprint("page_0 == page_1")
+                    #rprint(page[0] == page[1])
+                    if pre_msg == b"<END>" or page[0] == page[1]:
+                        # rprint("==========================")
+                        #bprint("Fin de mensaje")
+                        # rprint("==========================")
+                        t = False
+                        mlist.append(pre_msg)
+                        n_msgs_idx = 0
+                        #print("Cerrando", mlist)
+                        break
+                    else:
+                        mlist.append(pre_msg)
+                        #print("Added msg ", mlist)
+            except asyncio.CancelledError as ex:
+                self.logger.exception("Error de conexión cancelada  %s" %ex)                        
+                print("Asyncio cancelación de conexión")
+            except socket.error as ex:
+                self.logger.exception("Error de conexión de socket  %s" %ex)
+                print("Error de socket")
+                print("Falla en la conexión al recibir mensaje, %s"%ex)                
+            except Exception as ex:
+                self.logger.exception("Error de conexión  %s" %ex)            
+                print("Falla en la conexión al recibir mensaje, %s"%ex)
+                print("Lista msgs", mlist)
+            #print("Procesando final")
+            msg_tot = b"".join(mlist)
+            MSG_TOT = msg_tot.decode(char_code)
+            MT = MSG_TOT.strip()
+            MT = MSG_TOT.rstrip()
+            self.msg_r = MSG_TOT
+            return MSG_TOT
+        else:
+            return None
 
     def get_path(self):
         return self.gnc_path
@@ -627,13 +680,11 @@ class GNCSocket:
     async def set_reader_writer(self, reader, writer):
         idc = self.set_idc()
         # self.log_info_client(writer)
-        self.clients.update(
-            {idc: {
-                'reader': reader,
-                'writer': writer
-            }
-            }
-        )
+        new_client = {
+            'reader': reader,
+            'writer': writer
+        }
+        self.clients[idc]=new_client        
         return idc
 
     def log_info_client(self, writer):
@@ -706,7 +757,7 @@ class GNCSocket:
         reader = self.clients[idc]['reader']
         writer = self.clients[idc]['writer']
         writer.close()
-        await writer.wait_closed()        
+        await writer.wait_closed()
 
     async def create_server(self, callback_io, loop):
         # SI ES UNIX SOCKET
@@ -756,6 +807,14 @@ class GNCSocket:
                     print(self.server, type(self.server))
             else:
                 print("Asigna primero mode=server")
+        except asyncio.TimeoutError as te:
+            self.logger.exception("Tiempo fuera en intento de cerrar conexión %s, mode %s" %(
+                te, self.mode))
+            await asyncio.sleep(10)
+        except (ConnectionResetError, ConnectionAbortedError) as conn_error:
+            self.logger.exception("Excepción por desconexión %s, mode %s"%(
+                conn_error, self.mode))
+            await asyncio.sleep(10)
         except Exception as ex:
             print("Excepcion en create_server <::::", ex, "::::>")
         except asyncio.TimeoutError as te:
@@ -789,6 +848,14 @@ class GNCSocket:
                         ssl=self.context)
                     (reader, writer) = await asyncio.wait_for(future_client, timeout=self.timeout)
                     self.send_msg_sock_status(40)
+        except asyncio.TimeoutError as te:
+            self.logger.exception("Tiempo fuera en intento de cerrar conexión %s, mode %s" %(
+                te, self.mode))
+            await asyncio.sleep(10)
+        except (ConnectionResetError, ConnectionAbortedError) as conn_error:
+            self.logger.exception("Excepción por desconexión %s, mode %s"%(
+                conn_error, self.mode))
+            await asyncio.sleep(10)
         except Exception as ex:
             print("Excepcion en", ex)
             self.send_msg_sock_status(-1)
@@ -800,7 +867,10 @@ class GNCSocket:
         self.alert = "Conectado a socket-base"
         self.status = 'ON'
         print(self.alert)
+        print("Creando nuevo client")
         idc = await self.set_reader_writer(reader, writer)
+        print("Cliente creado--->", idc)
+        print("Clientes", self.clients)
         self.send_msg_sock_status(50)
         return idc
 
@@ -823,15 +893,8 @@ class GNCSocket:
         for i in range(len(self.conss)):
             print(str(self.addrs[i]) + ":" + str(self.conns[i]))
 
-    def close(self):
-        self.status = 'OFF'
-        [client.get('writer').close() for ids, client in self.clients.items()]
-        self.clean_socket(*self.address)
-        if self.mode == 'server':
-            if self.server:
-                self.server.close()
-
     def clean_client(self, idc):
+        print("Limpiando cliente", idc)
         if idc in self.clients:
             client = self.clients.get(idc)
             client.get('writer').close()
